@@ -3,15 +3,15 @@
 
 #include "HelpPage.hxx"
 #include "PageMeta.hxx"
-#include "ListPage.hxx"
-#include "ListRenderer.hxx"
-#include "ListText.hxx"
-#include "screen_find.hxx"
-#include "paint.hxx"
 #include "Bindings.hxx"
 #include "GlobalBindings.hxx"
 #include "config.h"
 #include "i18n.h"
+#include "screen.hxx"
+#include "page/ListPage.hxx"
+#include "ui/ListRenderer.hxx"
+#include "ui/ListText.hxx"
+#include "ui/paint.hxx"
 #include "util/LocaleString.hxx"
 
 #include <iterator>
@@ -25,18 +25,18 @@ struct HelpRow {
 
 	constexpr HelpRow(signed char _highlight,
 			  Command _command,
-			  const char *_text=nullptr)
+			  const char *_text=nullptr) noexcept
 		:highlight(_highlight), command(_command), text(_text) {}
 
 	constexpr HelpRow(Command _command,
-			  const char *_text=nullptr)
+			  const char *_text=nullptr) noexcept
 		:HelpRow(0, _command, _text) {}
 };
 
 static constexpr HelpRow HLINE{2, Command::NONE};
 
 static constexpr HelpRow
-Heading(const char *text)
+Heading(const char *text) noexcept
 {
 	return {1, Command::NONE, text};
 }
@@ -209,55 +209,55 @@ class HelpPage final : public ListPage, ListRenderer, ListText {
 	ScreenManager &screen;
 
 public:
-	HelpPage(ScreenManager &_screen, WINDOW *w, Size size)
-		:ListPage(w, size), screen(_screen) {
+	HelpPage(ScreenManager &_screen, const Window _window)
+		:ListPage(_screen, _window), screen(_screen) {
 		lw.HideCursor();
 		lw.SetLength(std::size(help_text));
 	}
 
 public:
 	/* virtual methods from class ListRenderer */
-	void PaintListItem(WINDOW *w, unsigned i,
+	void PaintListItem(const Window window, unsigned i,
 			   unsigned y, unsigned width,
 			   bool selected) const noexcept override;
 
 	/* virtual methods from class ListText */
-	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const noexcept override;
+	std::string_view GetListItemText(std::span<char> buffer,
+					 unsigned i) const noexcept override;
 
 	/* virtual methods from class Page */
 	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 
-	const char *GetTitle(char *, size_t) const noexcept override {
+	std::string_view GetTitle(std::span<char>) const noexcept override {
 		return _("Help");
 	}
 };
 
-const char *
-HelpPage::GetListItemText(char *, size_t, unsigned i) const noexcept
+std::string_view
+HelpPage::GetListItemText(std::span<char>, unsigned i) const noexcept
 {
 	const auto *row = &help_text[i];
 
 	assert(i < std::size(help_text));
 
 	if (row->text != nullptr)
-		return gettext(row->text);
+		return my_gettext(row->text);
 
 	if (row->command != Command::NONE)
 		return get_key_description(row->command);
 
-	return "";
+	return {};
 }
 
 static std::unique_ptr<Page>
-help_init(ScreenManager &screen, WINDOW *w, Size size)
+help_init(ScreenManager &screen, const Window window)
 {
-	return std::make_unique<HelpPage>(screen, w, size);
+	return std::make_unique<HelpPage>(screen, window);
 }
 
 void
-HelpPage::PaintListItem(WINDOW *w, unsigned i,
+HelpPage::PaintListItem(const Window window, unsigned i,
 			unsigned y, unsigned width,
 			bool selected) const noexcept
 {
@@ -265,28 +265,28 @@ HelpPage::PaintListItem(WINDOW *w, unsigned i,
 
 	assert(i < std::size(help_text));
 
-	row_color(w, row->highlight ? Style::LIST_BOLD : Style::LIST,
+	row_color(window, row->highlight ? Style::LIST_BOLD : Style::LIST,
 		  selected);
 
-	wclrtoeol(w);
+	window.ClearToEol();
 
 	if (row->command == Command::NONE) {
 		if (row->text != nullptr)
-			mvwaddstr(w, y, 6, gettext(row->text));
+			window.String({6, (int)y}, my_gettext(row->text));
 		else if (row->highlight == 2)
-			mvwhline(w, y, 3, ACS_HLINE, width - 6);
+			window.HLine({3, (int)y}, width - 6, ACS_HLINE);
 	} else {
 		const auto key =
 			GetGlobalKeyBindings().GetKeyNames(row->command);
 
-		if (StringWidthMB(key.c_str()) < 20)
-			wmove(w, y, 20 - StringWidthMB(key.c_str()));
-		waddstr(w, key.c_str());
-		mvwaddch(w, y, 21, ':');
-		mvwaddstr(w, y, 23,
-			  row->text != nullptr
-			  ? gettext(row->text)
-			  : get_key_description(row->command));
+		if (const auto key_width = StringWidthMB(key); key_width < 20)
+			window.MoveCursor({20 - (int)key_width, (int)y});
+		window.String(key);
+		window.Char({21, (int)y}, ':');
+		window.String({23, (int)y},
+			      row->text != nullptr
+			      ? my_gettext(row->text)
+			      : get_key_description(row->command));
 	}
 }
 
@@ -309,8 +309,8 @@ HelpPage::OnCommand(struct mpdclient &c, Command cmd)
 		   search */
 		lw.SetCursorFromOrigin(0);
 
-	if (screen_find(screen, lw, cmd, *this)) {
-		SetDirty();
+	if (screen.find_support.Find(lw, *this, cmd)) {
+		SchedulePaint();
 		return true;
 	}
 
