@@ -3,24 +3,27 @@
 
 #include "KeyDefPage.hxx"
 #include "PageMeta.hxx"
-#include "ListPage.hxx"
-#include "ListText.hxx"
-#include "TextListRenderer.hxx"
-#include "ProxyPage.hxx"
-#include "screen_status.hxx"
-#include "screen_find.hxx"
+#include "screen.hxx"
 #include "KeyName.hxx"
 #include "i18n.h"
 #include "ConfigFile.hxx"
 #include "Bindings.hxx"
 #include "GlobalBindings.hxx"
-#include "screen_utils.hxx"
 #include "Options.hxx"
+#include "page/ListPage.hxx"
+#include "page/ProxyPage.hxx"
+#include "dialogs/KeyDialog.hxx"
+#include "ui/Bell.hxx"
+#include "ui/ListText.hxx"
+#include "ui/TextListRenderer.hxx"
+#include "util/SPrintf.hxx"
 
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
 
+using std::string_view_literals::operator""sv;
+
 class CommandKeysPage final : public ListPage, ListText {
 	ScreenManager &screen;
 	Page *const parent;
@@ -38,9 +41,9 @@ class CommandKeysPage final : public ListPage, ListText {
 	unsigned n_keys = 0;
 
 public:
-	CommandKeysPage(ScreenManager &_screen, Page *_parent,
-			WINDOW *w, Size size) noexcept
-		:ListPage(w, size), screen(_screen), parent(_parent) {}
+	CommandKeysPage(PageContainer &_container, ScreenManager &_screen, Page *_parent,
+			const Window _window) noexcept
+		:ListPage(_container, _window), screen(_screen), parent(_parent) {}
 
 	void SetCommand(KeyBindings *_bindings, unsigned _cmd) {
 		bindings = _bindings;
@@ -96,7 +99,8 @@ private:
 	/**
 	 * Assigns a new key to a key slot.
 	 */
-	void OverwriteKey(int key_index);
+	[[nodiscard]]
+	Co::InvokeTask OverwriteKey(int key_index) noexcept;
 
 	/**
 	 * Assign a new key to a new slot.
@@ -108,12 +112,12 @@ public:
 	void OnOpen(struct mpdclient &c) noexcept override;
 	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const noexcept override;
+	std::string_view GetTitle(std::span<char> buffer) const noexcept override;
 
 private:
 	/* virtual methods from class ListText */
-	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const noexcept override;
+	std::string_view GetListItemText(std::span<char> buffer,
+					 unsigned i) const noexcept override;
 };
 
 /* TODO: rename to check_n_keys / subcmd_count_keys? */
@@ -141,55 +145,53 @@ CommandKeysPage::DeleteKey(int key_index)
 	binding->modified = true;
 	UpdateListLength();
 
-	screen_status_message(_("Deleted"));
+	Alert(_("Deleted"));
 
 	/* repaint */
-	SetDirty();
+	SchedulePaint();
 
 	/* update key conflict flags */
 	bindings->Check(nullptr, 0);
 }
 
-void
-CommandKeysPage::OverwriteKey(int key_index)
+Co::InvokeTask
+CommandKeysPage::OverwriteKey(int key_index) noexcept
 {
 	assert(key_index < MAX_COMMAND_KEYS);
 
-	char prompt[256];
-	snprintf(prompt, sizeof(prompt),
-		 _("Enter new key for %s: "),
-		 get_key_command_name(Command(subcmd)));
-	const int key = screen_getch(screen, prompt);
+	const auto prompt = fmt::format(fmt::runtime(_("Enter new key for {}")),
+					get_key_command_name(Command(subcmd)));
+	const int key = co_await KeyDialog{screen, prompt};
 
 	if (key == ERR) {
-		screen_status_message(_("Aborted"));
-		return;
+		Alert(_("Aborted"));
+		co_return;
 	}
 
 	if (key == '\0') {
-		screen_status_message(_("Ctrl-Space can't be used"));
-		return;
+		Alert(_("Ctrl-Space can't be used"));
+		co_return;
 	}
 
 	const Command cmd = bindings->FindKey(key);
 	if (cmd != Command::NONE) {
-		screen_status_printf(_("Error: key %s is already used for %s"),
-				     GetLocalizedKeyName(key),
-				     get_key_command_name(cmd));
-		screen_bell();
-		return;
+		FmtAlert(_("Error: key {} is already used for {}"),
+			 GetLocalizedKeyName(key),
+			 get_key_command_name(cmd));
+		Bell();
+		co_return;
 	}
 
 	binding->keys[key_index] = key;
 	binding->modified = true;
 
-	screen_status_printf(_("Assigned %s to %s"),
-			     GetLocalizedKeyName(key),
-			     get_key_command_name(Command(subcmd)));
+	FmtAlert(_("Assigned {} to {}"),
+		 GetLocalizedKeyName(key),
+		 get_key_command_name(Command(subcmd)));
 	UpdateListLength();
 
 	/* repaint */
-	SetDirty();
+	SchedulePaint();
 
 	/* update key conflict flags */
 	bindings->Check(nullptr, 0);
@@ -199,28 +201,24 @@ void
 CommandKeysPage::AddKey()
 {
 	if (n_keys < MAX_COMMAND_KEYS)
-		OverwriteKey(n_keys);
+		CoStart(OverwriteKey(n_keys));
 }
 
-const char *
-CommandKeysPage::GetListItemText(char *buffer, size_t size,
+std::string_view
+CommandKeysPage::GetListItemText(std::span<char> buffer,
 				 unsigned idx) const noexcept
 {
 	if (idx == GetLeavePosition())
-		return "[..]";
+		return "[..]"sv;
 
-	if (idx == GetAddPosition()) {
-		snprintf(buffer, size, "%d. %s", idx, _("Add new key"));
-		return buffer;
-	}
+	if (idx == GetAddPosition())
+		return SPrintf(buffer, "%d. %s", idx, _("Add new key"));
 
 	assert(IsKeyPosition(idx));
 
-	snprintf(buffer, size,
-		 "%d. %-20s   (%d) ", idx,
-		 GetLocalizedKeyName(binding->keys[PositionToKeyIndex(idx)]),
-		 binding->keys[PositionToKeyIndex(idx)]);
-	return buffer;
+	return SPrintf(buffer, "%d. %-20s   (%d) ", idx,
+		       GetLocalizedKeyName(binding->keys[PositionToKeyIndex(idx)]),
+		       binding->keys[PositionToKeyIndex(idx)]);
 }
 
 void
@@ -229,12 +227,11 @@ CommandKeysPage::OnOpen([[maybe_unused]] struct mpdclient &c) noexcept
 	// TODO
 }
 
-const char *
-CommandKeysPage::GetTitle(char *str, size_t size) const noexcept
+std::string_view
+CommandKeysPage::GetTitle(std::span<char> buffer) const noexcept
 {
-	snprintf(str, size, _("Edit keys for %s"),
-		 get_key_command_name(Command(subcmd)));
-	return str;
+	return SPrintf(buffer, _("Edit keys for %s"),
+		       get_key_command_name(Command(subcmd)));
 }
 
 void
@@ -262,7 +259,7 @@ CommandKeysPage::OnCommand(struct mpdclient &c, Command cmd)
 		} else {
 			/* just to be sure ;-) */
 			assert(IsKeyPosition(lw.GetCursorIndex()));
-			OverwriteKey(PositionToKeyIndex(lw.GetCursorIndex()));
+			CoStart(OverwriteKey(PositionToKeyIndex(lw.GetCursorIndex())));
 		}
 		return true;
 	case Command::DELETE:
@@ -277,8 +274,7 @@ CommandKeysPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, lw, cmd, *this);
-		SetDirty();
+		CoStart(screen.find_support.Find(lw, *this, cmd));
 		return true;
 
 	default:
@@ -295,8 +291,8 @@ class CommandListPage final : public ListPage, ListText {
 	static constexpr size_t command_n_commands = size_t(Command::NONE);
 
 public:
-	CommandListPage(ScreenManager &_screen, WINDOW *w, Size size)
-		:ListPage(w, size), screen(_screen) {}
+	CommandListPage(PageContainer &_container, ScreenManager &_screen, const Window _window)
+		:ListPage(_container, _window), screen(_screen) {}
 
 	~CommandListPage() override {
 		delete bindings;
@@ -348,12 +344,12 @@ public:
 	void OnOpen(struct mpdclient &c) noexcept override;
 	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const noexcept override;
+	std::string_view GetTitle(std::span<char> buffer) const noexcept override;
 
 private:
 	/* virtual methods from class ListText */
-	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const noexcept override;
+	std::string_view GetListItemText(std::span<char> buffer,
+					 unsigned i) const noexcept override;
 };
 
 bool
@@ -371,9 +367,9 @@ CommandListPage::Apply()
 	if (IsModified()) {
 		auto &orginal_bindings = GetGlobalKeyBindings();
 		orginal_bindings = *bindings;
-		screen_status_message(_("You have new key bindings"));
+		Alert(_("You have new key bindings"));
 	} else
-		screen_status_message(_("Keybindings unchanged."));
+		Alert(_("Keybindings unchanged."));
 }
 
 void
@@ -383,8 +379,8 @@ CommandListPage::Save()
 	if (options.key_file.empty()) {
 		filename = MakeKeysPath();
 		if (filename.empty()) {
-			screen_status_message(_("Unable to write configuration"));
-			screen_bell();
+			Alert(_("Unable to write configuration"));
+			Bell();
 			return;
 		}
 	} else
@@ -392,23 +388,22 @@ CommandListPage::Save()
 
 	FILE *f = fopen(filename.c_str(), "w");
 	if (f == nullptr) {
-		screen_status_printf("%s: %s - %s", _("Error"),
-				     filename.c_str(), strerror(errno));
-		screen_bell();
+		Alert(fmt::format("{}: {} - {}"sv, _("Error"),
+				  filename, strerror(errno)));
+		Bell();
 		return;
 	}
 
 	if (GetGlobalKeyBindings().WriteToFile(f, KEYDEF_WRITE_HEADER))
-		screen_status_printf(_("Wrote %s"), filename.c_str());
+		FmtAlert(_("Wrote {}"), filename);
 	else
-		screen_status_printf("%s: %s - %s", _("Error"),
-				     filename.c_str(), strerror(errno));
+		FmtAlert("{}: {} - {}"sv, _("Error"), filename, strerror(errno));
 
 	fclose(f);
 }
 
-const char *
-CommandListPage::GetListItemText(char *buffer, size_t size,
+std::string_view
+CommandListPage::GetListItemText(std::span<char> buffer,
 				 unsigned idx) const noexcept
 {
 	if (idx == command_item_apply())
@@ -427,16 +422,15 @@ CommandListPage::GetListItemText(char *buffer, size_t size,
 	 */
 	const char *name = get_key_command_name(Command(idx));
 	size_t len = strlen(name);
-	strncpy(buffer, name, size);
+	strncpy(buffer.data(), name, buffer.size());
 
 	if (len < get_cmds_max_name_width())
-		memset(buffer + len, ' ', get_cmds_max_name_width() - len);
+		memset(buffer.data() + len, ' ', get_cmds_max_name_width() - len);
 
-	snprintf(buffer + get_cmds_max_name_width(),
-		 size - get_cmds_max_name_width(),
-		 " - %s", gettext(get_command_definitions()[idx].description));
+	std::size_t length = SPrintf(buffer.subspan(get_cmds_max_name_width()),
+				     " - %s", my_gettext(get_command_definitions()[idx].description)).size();
 
-	return buffer;
+	return {buffer.data(), get_cmds_max_name_width() + length};
 }
 
 void
@@ -448,8 +442,8 @@ CommandListPage::OnOpen([[maybe_unused]] struct mpdclient &c) noexcept
 	lw.SetLength(command_length());
 }
 
-const char *
-CommandListPage::GetTitle(char *, size_t) const noexcept
+std::string_view
+CommandListPage::GetTitle(std::span<char>) const noexcept
 {
 	return _("Edit key bindings");
 }
@@ -486,8 +480,7 @@ CommandListPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, lw, cmd, *this);
-		SetDirty();
+		CoStart(screen.find_support.Find(lw, *this, cmd));
 		return true;
 
 	default:
@@ -502,10 +495,10 @@ class KeyDefPage final : public ProxyPage {
 	CommandKeysPage command_keys_page;
 
 public:
-	KeyDefPage(ScreenManager &screen, WINDOW *_w, Size size)
-		:ProxyPage(_w),
-		 command_list_page(screen, _w, size),
-		 command_keys_page(screen, this, _w, size) {}
+	KeyDefPage(ScreenManager &screen, const Window _window)
+		:ProxyPage(screen, _window),
+		 command_list_page(*this, screen, _window),
+		 command_keys_page(*this, screen, this, _window) {}
 
 public:
 	/* virtual methods from class Page */
@@ -515,9 +508,9 @@ public:
 };
 
 static std::unique_ptr<Page>
-keydef_init(ScreenManager &screen, WINDOW *w, Size size)
+keydef_init(ScreenManager &screen, const Window window)
 {
-	return std::make_unique<KeyDefPage>(screen, w, size);
+	return std::make_unique<KeyDefPage>(screen, window);
 }
 
 void
@@ -533,7 +526,7 @@ void
 KeyDefPage::OnClose() noexcept
 {
 	if (command_list_page.IsModified())
-		screen_status_message(_("Note: Did you forget to \'Apply\' your changes?"));
+		Alert(_("Note: Did you forget to \'Apply\' your changes?"));
 
 	ProxyPage::OnClose();
 }
