@@ -3,7 +3,7 @@
 
 #include "ProgressBar.hxx"
 #include "Styles.hxx"
-#include "Options.hxx"
+#include "ui/Options.hxx"
 #include "config.h"
 
 #include <assert.h>
@@ -12,37 +12,43 @@ ProgressBar::ProgressBar(Point p, unsigned _width) noexcept
 	:window(p, {_width, 1u})
 {
 	leaveok(window.w, true);
-#ifdef ENABLE_COLORS
-	if (options.enable_colors)
+
+	if (ui_options.enable_colors)
 		window.SetBackgroundStyle(Style::PROGRESSBAR);
-#endif
 }
 
 void
 ProgressBar::Paint() const noexcept
 {
+	const unsigned window_width = window.GetWidth();
+
 	if (max > 0) {
-		assert(width < window.size.width);
+		assert(width < window_width);
 
-		SelectStyle(window.w, Style::PROGRESSBAR);
+		SelectStyle(window, Style::PROGRESSBAR);
 
-		if (width > 0)
-			mvwhline(window.w, 0, 0, '=', width);
+		if (width > 0) {
+#if NCURSES_WIDECHAR
+			window.HLine({0, 0}, width, WACS_D_HLINE);
+#else
+			window.HLine({0, 0}, width, '=');
+#endif
+		}
 
-		mvwaddch(window.w, 0, width, '>');
+		window.Char({(int)width, 0}, '>');
 		unsigned x = width + 1;
 
-		if (x < window.size.width) {
-			SelectStyle(window.w, Style::PROGRESSBAR_BACKGROUND);
-			mvwhline(window.w, 0, x, ACS_HLINE, window.size.width - x);
+		if (x < window_width) {
+			SelectStyle(window, Style::PROGRESSBAR_BACKGROUND);
+			window.HLine({(int)x, 0}, window_width - x, ACS_HLINE);
 		}
 	} else {
 		/* no progress bar, just a simple horizontal line */
-		SelectStyle(window.w, Style::LINE);
-		mvwhline(window.w, 0, 0, ACS_HLINE, window.size.width);
+		SelectStyle(window, Style::LINE);
+		window.HLine({0, 0}, window_width, ACS_HLINE);
 	}
 
-	wnoutrefresh(window.w);
+	window.RefreshNoOut();
 }
 
 bool
@@ -51,9 +57,10 @@ ProgressBar::Calculate() noexcept
 	if (max == 0)
 		return false;
 
+	const unsigned window_width = window.GetWidth();
 	unsigned old_width = width;
-	width = (window.size.width * current) / (max + 1);
-	assert(width < window.size.width);
+	width = (window_width * current) / (max + 1);
+	assert(width < window_width);
 
 	return width != old_width;
 }
