@@ -3,7 +3,6 @@
 
 #include "QueuePage.hxx"
 #include "PageMeta.hxx"
-#include "screen_status.hxx"
 #include "save_playlist.hxx"
 #include "config.h"
 #include "i18n.h"
@@ -308,8 +307,7 @@ handle_add_to_playlist(ScreenManager &screen, struct mpdclient &c)
 		screen,
 		_("Add"),
 		{},
-		nullptr,
-		completion,
+		{ .completion = completion },
 	};
 
 	/* add the path to the playlist */
