@@ -50,24 +50,18 @@ static constexpr struct tag_label tag_labels[] = {
 	{ MPD_TAG_ARTIST, N_("Artist") },
 	{ MPD_TAG_TITLE, N_("Title") },
 	{ MPD_TAG_ALBUM, N_("Album") },
-#if LIBMPDCLIENT_CHECK_VERSION(2,17,0)
 	{ MPD_TAG_WORK, N_("Work") },
-#endif
 	{ LABEL_LENGTH, N_("Length") },
 	{ LABEL_POSITION, N_("Position") },
 	{ MPD_TAG_COMPOSER, N_("Composer") },
 	{ MPD_TAG_PERFORMER, N_("Performer") },
-#if LIBMPDCLIENT_CHECK_VERSION(2,17,0)
 	{ MPD_TAG_CONDUCTOR, N_("Conductor") },
-#endif
 	{ MPD_TAG_NAME, N_("Name") },
 	{ MPD_TAG_DISC, N_("Disc") },
 	{ MPD_TAG_TRACK, N_("Track") },
 	{ MPD_TAG_DATE, N_("Date") },
 	{ MPD_TAG_GENRE, N_("Genre") },
-#if LIBMPDCLIENT_CHECK_VERSION(2,17,0)
 	{ MPD_TAG_LABEL, N_("Label") },
-#endif
 	{ MPD_TAG_COMMENT, N_("Comment") },
 	{ LABEL_PATH, N_("Path") },
 	{ LABEL_BITRATE, N_("Bitrate") },
@@ -289,7 +283,7 @@ SongPage::AppendTag(const struct mpd_song *song,
 	unsigned i = 0;
 	const char *value;
 
-	assert((unsigned)tag < std::size(tag_labels));
+	assert(tag < MPD_TAG_COUNT);
 	assert(label != nullptr);
 
 	while ((value = mpd_song_get_tag(song, tag, i++)) != nullptr)
