@@ -8,27 +8,29 @@
 #include "strfsong.hxx"
 #include "DelayedSeek.hxx"
 #include "time_format.hxx"
+#include "ui/Options.hxx"
+#include "lib/fmt/ToSpan.hxx"
 #include "util/LocaleString.hxx"
 
 #include <mpd/client.h>
 
 #include <string.h>
 
+using std::string_view_literals::operator""sv;
+
 StatusBar::StatusBar(EventLoop &event_loop,
 		     Point p, unsigned width) noexcept
 	:window(p, {width, 1u}),
 	 message_timer(event_loop, BIND_THIS_METHOD(OnMessageTimer))
 #ifndef NCMPC_MINI
-	, hscroll(event_loop, window.w, options.scroll_sep.c_str())
+	, hscroll(event_loop, window, options.scroll_sep)
 #endif
 {
 	leaveok(window.w, false);
 	keypad(window.w, true);
 
-#ifdef ENABLE_COLORS
-	if (options.enable_colors)
+	if (ui_options.enable_colors)
 		window.SetBackgroundStyle(Style::STATUS);
-#endif
 }
 
 StatusBar::~StatusBar() noexcept
@@ -49,36 +51,35 @@ StatusBar::ClearMessage() noexcept
 	doupdate();
 }
 
-static size_t
-format_bitrate(char *p, size_t max_length,
+[[nodiscard]]
+static std::string_view 
+format_bitrate(std::span<char> buffer,
 	       const struct mpd_status *status) noexcept
 {
 
 #ifndef NCMPC_MINI
 	if (options.visible_bitrate && mpd_status_get_kbit_rate(status) > 0) {
-		snprintf(p, max_length,
-			 " [%d kbps]",
-			 mpd_status_get_kbit_rate(status));
-		return strlen(p);
+		return FmtTruncate(buffer, " [{} kbps]",
+				   mpd_status_get_kbit_rate(status));
 	} else {
 #else
-		(void)max_length;
+		(void)buffer;
 		(void)status;
 #endif
-		p[0] = '\0';
-		return 0;
+		return {};
 #ifndef NCMPC_MINI
 	}
 #endif
 }
 
-static void
-FormatCurrentSongTime(char *buffer, size_t size,
+[[nodiscard]]
+static std::string_view
+FormatCurrentSongTime(std::span<char> buffer,
 		      const struct mpd_status &status,
 		      const DelayedSeek &seek) noexcept
 {
 	if (options.current_time_display == CurrentTimeDisplay::NONE)
-		return;
+		return {};
 
 	const unsigned total_time = mpd_status_get_total_time(&status);
 
@@ -86,7 +87,7 @@ FormatCurrentSongTime(char *buffer, size_t size,
 		? seek.GetTime()
 		: mpd_status_get_elapsed_time(&status);
 
-	char elapsed_string[32], duration_string[32];
+	char elapsed_buffer[32], duration_buffer[32];
 
 	/* checks the conf to see whether to display elapsed or
 	   remaining time */
@@ -97,7 +98,7 @@ FormatCurrentSongTime(char *buffer, size_t size,
 
 	case CurrentTimeDisplay::REMAINING:
 		if (total_time == 0)
-			return;
+			return {};
 
 		elapsed_time = elapsed_time < total_time
 			? total_time - elapsed_time
@@ -106,33 +107,27 @@ FormatCurrentSongTime(char *buffer, size_t size,
 	}
 
 	/* write out the time */
-	format_duration_short(elapsed_string,
-			      sizeof(elapsed_string),
-			      elapsed_time);
+	const auto elapsed_string = format_duration_short(elapsed_buffer, elapsed_time);
 
-	if (total_time == 0) {
-		snprintf(buffer, size, " [%s]", elapsed_string);
-		return;
-	}
+	if (total_time == 0)
+		return FmtTruncate(buffer, " [{}]"sv, elapsed_string);
 
-	format_duration_short(duration_string,
-			      sizeof(duration_string),
-			      total_time);
-
-	snprintf(buffer, size, " [%s/%s]", elapsed_string, duration_string);
+	const auto duration_string = format_duration_short(duration_buffer, total_time);
+	return FmtTruncate(buffer, " [{}/{}]"sv, elapsed_string, duration_string);
 }
 
-inline size_t
-FormatStatusRightText(char *buffer, size_t size,
+[[nodiscard]]
+static std::string_view
+FormatStatusRightText(std::span<char> buffer,
 		      const struct mpd_status &status,
 		      const DelayedSeek &seek) noexcept
 {
 	/* display bitrate if visible-bitrate is true */
-	size_t offset = format_bitrate(buffer, size, &status);
+	const auto bitrate_string = format_bitrate(buffer, &status);
 
-	FormatCurrentSongTime(buffer + offset, size - offset, status, seek);
+	const auto time_string = FormatCurrentSongTime(buffer.subspan(bitrate_string.size()), status, seek);
 
-	return StringWidthMB(buffer);
+	return {buffer.data(), bitrate_string.size() + time_string.size()};
 }
 
 void
@@ -164,30 +159,26 @@ StatusBar::Update(const struct mpd_status *status,
 		: 0;
 
 	if (state == MPD_STATE_PLAY || state == MPD_STATE_PAUSE) {
-		right_width = FormatStatusRightText(right_text,
-						    sizeof(right_text),
-						    *status, seek);
-
-#ifndef NCMPC_MINI
-		int width = COLS - left_width - right_width;
-#endif
+		const auto right_text = FormatStatusRightText(right_buffer,
+							      *status, seek);
+		right_length = right_text.size();
+		right_width = StringWidthMB(right_text);
 
 		if (song) {
 			char buffer[1024];
-			strfsong(buffer, sizeof(buffer),
-				 options.status_format.c_str(), song);
-			center_text = buffer;
+			center_text = strfsong(buffer, options.status_format.c_str(), *song);
 		} else
 			center_text.clear();
 
 		/* scroll if the song name is to long */
 #ifndef NCMPC_MINI
 		if (options.scroll) {
+			const unsigned window_width = window.GetWidth();
 			const unsigned center_width =
-				StringWidthMB(center_text.c_str());
-			if (width > 3 && center_width > (unsigned)width) {
-				hscroll.Set(left_width, 0, width,
-					    center_text.c_str(),
+				StringWidthMB(center_text);
+			if (window_width > 3 && center_width > window_width) {
+				hscroll.Set({(int)left_width, 0}, window_width,
+					    center_text,
 					    Style::STATUS);
 			} else {
 				hscroll.Clear();
@@ -209,35 +200,35 @@ StatusBar::Update(const struct mpd_status *status,
 void
 StatusBar::Paint() const noexcept
 {
-	WINDOW *w = window.w;
+	const unsigned window_width = window.GetWidth();
 
-	wmove(w, 0, 0);
-	wclrtoeol(w);
+	window.MoveCursor({0, 0});
+	window.ClearToEol();
 
 	if (!message.empty()) {
-		SelectStyle(w, Style::STATUS_ALERT);
-		waddstr(w, message.c_str());
-		wnoutrefresh(w);
+		SelectStyle(window, Style::STATUS_ALERT);
+		window.String(message);
+		window.RefreshNoOut();
 		return;
 	}
 
-	SelectStyle(w, Style::STATUS_BOLD);
+	SelectStyle(window, Style::STATUS_BOLD);
 
 	if (left_text != nullptr)
 		/* display state */
-		waddstr(w, left_text);
+		window.String(left_text);
 
 	if (right_width > 0) {
 		/* display time string */
-		int x = window.size.width - right_width;
-		SelectStyle(w, Style::STATUS_TIME);
-		mvwaddstr(w, 0, x, right_text);
+		int x = window_width - right_width;
+		SelectStyle(window, Style::STATUS_TIME);
+		window.String({x, 0}, {right_buffer, right_length});
 	}
 
 	if (!center_text.empty()) {
 		/* display song name */
 
-		SelectStyle(w, Style::STATUS);
+		SelectStyle(window, Style::STATUS);
 
 		/* scroll if the song name is to long */
 #ifndef NCMPC_MINI
@@ -245,15 +236,10 @@ StatusBar::Paint() const noexcept
 			hscroll.Paint();
 		else
 #endif
-			mvwaddstr(w, 0, left_width, center_text.c_str());
+			window.String({(int)left_width, 0}, center_text);
 	}
 
-	/* display time string */
-	int x = window.size.width - right_width;
-	SelectStyle(w, Style::STATUS_TIME);
-	mvwaddstr(w, 0, x, right_text);
-
-	wnoutrefresh(w);
+	window.RefreshNoOut();
 }
 
 void
@@ -264,14 +250,14 @@ StatusBar::OnResize(Point p, unsigned width) noexcept
 }
 
 void
-StatusBar::SetMessage(const char *msg) noexcept
+StatusBar::SetMessage(std::string &&msg) noexcept
 {
 #ifndef NCMPC_MINI
 	if (options.scroll)
 		hscroll.Clear();
 #endif
 
-	message = msg;
+	message = std::move(msg);
 	Paint();
 	doupdate();
 
