@@ -5,6 +5,7 @@
 #include "config.h"
 #include "gidle.hxx"
 #include "charset.hxx"
+#include "util/ScopeExit.hxx"
 
 #include <mpd/client.h>
 
@@ -195,9 +196,9 @@ mpdclient::GetSettingsName() const noexcept
 	if (settings == nullptr)
 		return "unknown";
 
-	auto name = settings_name(settings);
-	mpd_settings_free(settings);
-	return name;
+	AtScopeExit(settings) { mpd_settings_free(settings); };
+
+	return settings_name(settings);
 #endif
 }
 
@@ -290,9 +291,7 @@ static TagMask
 ResetAndObtainTagList(struct mpd_connection *c) noexcept
 {
 	if (!mpd_command_list_begin(c, false) ||
-#if LIBMPDCLIENT_CHECK_VERSION(2,19,0)
 	    !mpd_send_all_tag_types(c) ||
-#endif
 	    !mpd_send_list_tag_types(c) ||
 	    !mpd_command_list_end(c))
 		return TagMask::None();
