@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SignalMonitor.hxx"
 #include "event/Features.h"
@@ -37,6 +21,7 @@
 #endif
 
 #include <algorithm>
+#include <array>
 #include <cassert>
 #include <csignal>
 
@@ -62,7 +47,7 @@ public:
 #endif
 	}
 
-	auto &GetEventLoop() const noexcept {
+	[[nodiscard]] auto &GetEventLoop() const noexcept {
 		return event.GetEventLoop();
 	}
 
@@ -90,12 +75,12 @@ private:
 /* this should be enough - is it? */
 static constexpr unsigned MAX_SIGNAL = 64;
 
-static SignalHandler signal_handlers[MAX_SIGNAL];
+static std::array<SignalHandler, MAX_SIGNAL> signal_handlers;
 
 #ifdef USE_SIGNALFD
 static sigset_t signal_mask;
 #else
-static std::atomic_bool signal_pending[MAX_SIGNAL];
+static std::array<std::atomic_bool, MAX_SIGNAL> signal_pending;
 #endif
 
 static Manual<SignalMonitor> monitor;
@@ -153,7 +138,7 @@ void
 SignalMonitorFinish() noexcept
 {
 #ifdef USE_SIGNALFD
-	std::fill_n(signal_handlers, MAX_SIGNAL, nullptr);
+	signal_handlers = {};
 #else
 	struct sigaction sa;
 	sa.sa_flags = 0;
@@ -167,7 +152,7 @@ SignalMonitorFinish() noexcept
 		}
 	}
 
-	std::fill_n(signal_pending, MAX_SIGNAL, false);
+	std::fill(signal_pending.begin(), signal_pending.end(), false);
 #endif
 
 	monitor.Destruct();
