@@ -1,29 +1,14 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "filelist.hxx"
+#include "util/StringAPI.hxx"
 #include "util/StringUTF8.hxx"
 
 #include <mpd/client.h>
 
 #include <algorithm>
 
-#include <string.h>
 #include <assert.h>
 
 FileListEntry::~FileListEntry() noexcept
@@ -120,7 +105,7 @@ FileList::RemoveDuplicateSongs() noexcept
 static bool
 same_song(const struct mpd_song *a, const struct mpd_song *b) noexcept
 {
-	return strcmp(mpd_song_get_uri(a), mpd_song_get_uri(b)) == 0;
+	return StringIsEqual(mpd_song_get_uri(a), mpd_song_get_uri(b));
 }
 
 int
@@ -152,8 +137,8 @@ FileList::FindDirectory(const char *name) const noexcept
 
 		if (entity != nullptr &&
 		    mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_DIRECTORY &&
-		    strcmp(mpd_directory_get_path(mpd_entity_get_directory(entity)),
-			   name) == 0)
+		    StringIsEqual(mpd_directory_get_path(mpd_entity_get_directory(entity)),
+				  name))
 			return i;
 	}
 
