@@ -6,10 +6,13 @@
 #include "StaticSocketAddress.hxx"
 #include "IPv4Address.hxx"
 #include "IPv6Address.hxx"
+#include "UniqueSocketDescriptor.hxx"
 
 #ifdef _WIN32
 #include <ws2tcpip.h>
 #else
+#include "MsgHdr.hxx"
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -23,14 +26,7 @@
 int
 SocketDescriptor::GetType() const noexcept
 {
-	assert(IsDefined());
-
-	int type;
-	socklen_t size = sizeof(type);
-	return getsockopt(fd, SOL_SOCKET, SO_TYPE,
-			  (char *)&type, &size) == 0
-		? type
-		: -1;
+	return GetIntOption(SOL_SOCKET, SO_TYPE, -1);
 }
 
 bool
@@ -39,6 +35,16 @@ SocketDescriptor::IsStream() const noexcept
 	return GetType() == SOCK_STREAM;
 }
 
+#ifdef __linux__
+
+int
+SocketDescriptor::GetProtocol() const noexcept
+{
+	return GetIntOption(SOL_SOCKET, SO_PROTOCOL, -1);
+}
+
+#endif // __linux__
+
 #ifdef _WIN32
 
 void
@@ -188,12 +194,9 @@ SocketDescriptor::CreateSocketPairNonBlock(int domain, int type, int protocol,
 int
 SocketDescriptor::GetError() const noexcept
 {
-	assert(IsDefined());
-
 	int s_err = 0;
-	socklen_t s_err_size = sizeof(s_err);
-	return getsockopt(fd, SOL_SOCKET, SO_ERROR,
-			  (char *)&s_err, &s_err_size) == 0
+	return GetOption(SOL_SOCKET, SO_ERROR,
+			 &s_err, sizeof(s_err)) == sizeof(s_err)
 		? s_err
 		: errno;
 }
@@ -210,6 +213,14 @@ SocketDescriptor::GetOption(int level, int name,
 		: 0;
 }
 
+int
+SocketDescriptor::GetIntOption(int level, int name, int fallback) const noexcept
+{
+	int value = fallback;
+	(void)GetOption(level, name, &value, sizeof(value));
+	return value;
+}
+
 #ifdef HAVE_STRUCT_UCRED
 
 struct ucred
@@ -233,7 +244,15 @@ SocketDescriptor::SetNonBlocking() const noexcept
 	return ioctlsocket(fd, FIONBIO, &val) == 0;
 }
 
-#endif
+#else
+
+UniqueSocketDescriptor
+SocketDescriptor::Duplicate() const noexcept
+{
+	return UniqueSocketDescriptor{FileDescriptor::Duplicate()};
+}
+
+#endif // !_WIN32
 
 bool
 SocketDescriptor::SetOption(int level, int name,
@@ -404,25 +423,81 @@ SocketDescriptor::GetPeerAddress() const noexcept
 }
 
 ssize_t
-SocketDescriptor::Read(void *buffer, std::size_t length) const noexcept
+SocketDescriptor::Receive(std::span<std::byte> dest, int flags) const noexcept
+{
+	return ::recv(Get(), (char *)dest.data(), dest.size(), flags);
+}
+
+#ifndef _WIN32
+
+ssize_t
+SocketDescriptor::Receive(struct msghdr &msg, int flags) const noexcept
+{
+	return ::recvmsg(Get(), &msg, flags);
+}
+
+ssize_t
+SocketDescriptor::Receive(std::span<const struct iovec> v, int flags) const noexcept
+{
+	auto msg = MakeMsgHdr(v);
+	return Receive(msg, flags);
+}
+
+#endif // !_WIN32
+
+ssize_t
+SocketDescriptor::Send(std::span<const std::byte> src, int flags) const noexcept
+{
+#ifdef __linux__
+	flags |= MSG_NOSIGNAL;
+#endif
+
+	return ::send(Get(), (const char *)src.data(), src.size(), flags);
+}
+
+#ifndef _WIN32
+
+ssize_t
+SocketDescriptor::Send(const struct msghdr &msg, int flags) const noexcept
+{
+#ifdef __linux__
+	flags |= MSG_NOSIGNAL;
+#endif
+
+	return ::sendmsg(Get(), &msg, flags);
+}
+
+ssize_t
+SocketDescriptor::Send(std::span<const struct iovec> v, int flags) const noexcept
+{
+	return Send(MakeMsgHdr(v), flags);
+}
+
+#endif // !_WIN32
+
+ssize_t
+SocketDescriptor::ReadNoWait(std::span<std::byte> dest) const noexcept
 {
 	int flags = 0;
 #ifndef _WIN32
 	flags |= MSG_DONTWAIT;
 #endif
 
-	return ::recv(Get(), (char *)buffer, length, flags);
+	return Receive(dest, flags);
 }
 
 ssize_t
-SocketDescriptor::Write(const void *buffer, std::size_t length) const noexcept
+SocketDescriptor::WriteNoWait(std::span<const std::byte> src) const noexcept
 {
 	int flags = 0;
+#ifndef _WIN32
+	flags |= MSG_DONTWAIT;
+#endif
 #ifdef __linux__
 	flags |= MSG_NOSIGNAL;
 #endif
 
-	return ::send(Get(), (const char *)buffer, length, flags);
+	return Send(src, flags);
 }
 
 #ifdef _WIN32
@@ -468,8 +543,8 @@ SocketDescriptor::WaitWritable(int timeout_ms) const noexcept
 #endif
 
 ssize_t
-SocketDescriptor::Read(void *buffer, std::size_t length,
-		       StaticSocketAddress &address) const noexcept
+SocketDescriptor::ReadNoWait(std::span<std::byte> dest,
+			     StaticSocketAddress &address) const noexcept
 {
 	int flags = 0;
 #ifndef _WIN32
@@ -477,7 +552,9 @@ SocketDescriptor::Read(void *buffer, std::size_t length,
 #endif
 
 	socklen_t addrlen = address.GetCapacity();
-	ssize_t nbytes = ::recvfrom(Get(), (char *)buffer, length, flags,
+	ssize_t nbytes = ::recvfrom(Get(),
+				    reinterpret_cast<char *>(dest.data()),
+				    dest.size(), flags,
 				    address, &addrlen);
 	if (nbytes > 0)
 		address.SetSize(addrlen);
@@ -486,8 +563,8 @@ SocketDescriptor::Read(void *buffer, std::size_t length,
 }
 
 ssize_t
-SocketDescriptor::Write(const void *buffer, std::size_t length,
-			SocketAddress address) const noexcept
+SocketDescriptor::WriteNoWait(std::span<const std::byte> src,
+			      SocketAddress address) const noexcept
 {
 	int flags = 0;
 #ifndef _WIN32
@@ -497,7 +574,8 @@ SocketDescriptor::Write(const void *buffer, std::size_t length,
 	flags |= MSG_NOSIGNAL;
 #endif
 
-	return ::sendto(Get(), (const char *)buffer, length, flags,
+	return ::sendto(Get(), reinterpret_cast<const char *>(src.data()),
+			src.size(), flags,
 			address.GetAddress(), address.GetSize());
 }
 
