@@ -1,26 +1,12 @@
-/* ncmpc (Ncurses MPD Client)
- * Copyright 2004-2021 The Music Player Daemon Project
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "plugin.hxx"
 #include "io/Path.hxx"
 #include "io/UniqueFileDescriptor.hxx"
 #include "event/PipeEvent.hxx"
 #include "event/CoarseTimerEvent.hxx"
+#include "event/Features.h" // for USE_SIGNALFD
 #include "util/ScopeExit.hxx"
 #include "util/UriUtil.hxx"
 
@@ -29,12 +15,17 @@
 
 #include <assert.h>
 #include <stdlib.h>
+#include <spawn.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <signal.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+#ifdef __APPLE__
+extern char **environ;
+#endif
+
 struct PluginCycle;
 
 class PluginPipe {
@@ -263,28 +254,28 @@ PluginCycle::LaunchPlugin(const char *plugin_path) noexcept
 	    !UniqueFileDescriptor::CreatePipe(stderr_r, stderr_w))
 		return -1;
 
-	pid = fork();
+	posix_spawn_file_actions_t file_actions;
+	posix_spawn_file_actions_init(&file_actions);
+	AtScopeExit(&file_actions) { posix_spawn_file_actions_destroy(&file_actions); };
 
-	if (pid < 0)
+	posix_spawn_file_actions_addclose(&file_actions, STDIN_FILENO);
+	posix_spawn_file_actions_adddup2(&file_actions, stdout_w.Get(),
+					 STDOUT_FILENO);
+	posix_spawn_file_actions_adddup2(&file_actions, stderr_w.Get(),
+					 STDERR_FILENO);
+
+	posix_spawnattr_t attr;
+	posix_spawnattr_init(&attr);
+
+#ifdef USE_SIGNALFD
+	/* unblock all signals which may be blocked for signalfd */
+	posix_spawnattr_setflags(&attr, POSIX_SPAWN_SETSIGMASK);
+#endif
+
+	if (posix_spawn(&pid, plugin_path, &file_actions, &attr,
+			argv.get(), environ) != 0)
 		return -1;
 
-	if (pid == 0) {
-		stdout_w.Duplicate(FileDescriptor(STDOUT_FILENO));
-		stderr_w.Duplicate(FileDescriptor(STDERR_FILENO));
-
-		stdout_r.Close();
-		stdout_w.Close();
-		stderr_r.Close();
-		stderr_w.Close();
-		close(0);
-		/* XXX close other fds? */
-
-		execv(plugin_path, argv.get());
-		_exit(1);
-	}
-
-	/* XXX CLOEXEC? */
-
 	pipe_stdout.Start(std::move(stdout_r));
 	pipe_stderr.Start(std::move(stderr_r));
 
